@@ -143,7 +143,7 @@ class Device(Events):
     def push_watcher_data(self):
         self.__main_conn.send({'watcher_list': self.watcher_list})
 
-    def watcher(self, name: str):
+    def watcher(self, name: str) -> Watcher:
         w = Watcher({'name': name}, self)
         return w
 
@@ -173,17 +173,17 @@ class Device(Events):
                 syslog_file.write(log_str + "\n")
                 syslog_file.flush()
 
-    def set_syslog_output(self, is_enable, keyword=""):
+    def set_syslog_output(self, is_enable: bool, keyword: str = "") -> None:
         self.__syslog_output_keyword = keyword
         self.__syslog_output = is_enable
 
-    def syslog_output(self):
+    def syslog_output(self) -> bool:
         return self.__syslog_output
 
-    def syslog_output_keyword(self):
+    def syslog_output_keyword(self) -> str:
         return self.__syslog_output_keyword
 
-    def set_syslog_save_start(self, save_path="./syslog/", save_name=None, save_keyword=""):
+    def set_syslog_save_start(self, save_path: str = "./syslog/", save_name: str = None, save_keyword: str = "") -> None:
         self.__syslog_save_path = save_path
         if save_name is None:
             current_remote_time = self.con.get(path="getSystemTime").read()
@@ -191,31 +191,31 @@ class Device(Events):
         self.__syslog_save_keyword = save_keyword
         self.__syslog_save = True
 
-    def set_syslog_save_stop(self):
+    def set_syslog_save_stop(self) -> None:
         self.__syslog_save = False
 
-    def syslog_save(self):
+    def syslog_save(self) -> bool:
         return self.__syslog_save
 
-    def syslog_save_path(self):
+    def syslog_save_path(self) -> str:
         return self.__syslog_save_path
 
-    def syslog_save_name(self):
+    def syslog_save_name(self) -> str:
         return self.__syslog_save_name
 
-    def syslog_save_keyword(self):
+    def syslog_save_keyword(self) -> str:
         return self.__syslog_save_keyword
 
-    def set_default_timeout(self, timeout):
+    def set_default_timeout(self, timeout: int) -> None:
         self.default_timeout = timeout
 
-    def set_xpath_list(self, path):
+    def set_xpath_list(self, path: str) -> None:
         self.__xpath_file = path
 
-    def set_screenshots_path(self, path):
+    def set_screenshots_path(self, path: str) -> None:
         self.__screenshots = path
 
-    def screenshot(self, path=__screenshots):
+    def screenshot(self, path: str = __screenshots) -> str:
         if not os.path.exists(path):
             os.makedirs(path)
         img_base64 = str(self.con.get(path="getScreenShot").read(), 'utf-8').split(',')[0]
@@ -226,18 +226,26 @@ class Device(Events):
         image.close()
         return file_name
 
+    def grab_image_to_base64(self, x: int, y: int, width: int, height: int, rotation: int = 0, scale: float = 1) -> str:
+        return str(self.con.get(path="grabImage?x=" + str(round(x))
+                              + "&y=" + str(round(y))
+                              + "&w=" + str(round(width))
+                              + "&h=" + str(round(height))
+                              + "&r=" + str(round(rotation))
+                              + "&s=" + str(round(scale))).read(), 'utf-8')
+
     def __set_display_size(self):
         image_data = str(self.con.get(path="getScreenShot").read(), 'utf-8')
         self.__height = int(image_data.split(",")[1])
         self.__width = int(image_data.split(",")[2])
 
-    def display_width(self):
+    def display_width(self) -> int:
         return self.__width
 
-    def display_height(self):
+    def display_height(self) -> int:
         return self.__height
 
-    def get_xpath(self, sop_id, key):
+    def get_xpath(self, sop_id: str, key: str) -> str:
         if not os.path.exists(self.__xpath_file):
             f = open(self.__xpath_file, "w")
             f.close()
@@ -245,19 +253,19 @@ class Device(Events):
         conf.read(self.__xpath_file)
         return conf.get(sop_id, key)
 
-    def refresh_layout(self):
+    def refresh_layout(self) -> None:
         self.xmlStr = str(self.con.get(path="getLayoutXML").read(), 'utf-8')
 
-    def os_version(self):
+    def os_version(self) -> str:
         return self.__osVersion
 
-    def serial_number(self):
+    def serial_number(self) -> str:
         return self.__serialNumber
     
-    def find_item_by_xpath_key(self, sopid, xpath_key):
+    def find_item_by_xpath_key(self, sopid: str, xpath_key: str) -> Item:
         i = Item(d=self, s=sopid, xpath=self.get_xpath(sopid, xpath_key))
         return i
 
-    def find_item_by_xpath(self, sopid, xpath):
+    def find_item_by_xpath(self, sopid: str, xpath: str) -> Item:
         i = Item(d=self, s=sopid, xpath=xpath)
         return i
