@@ -26,6 +26,7 @@ import numpy as np
 import random
 from time import sleep
 import io
+from .DataStruct import *
 
 
 html_string_1 = "<!DOCTYPE html><html lang=\"en\"><head><meta charset=\"UTF-8\"><title>render</title></head><body style=\"background:#000;clip:rect(auto,auto,auto,auto);\"><div id=\"content\" style=\"width:"
@@ -79,15 +80,15 @@ class Item:
             self.__init_attribute()
 
     def __refresh_attribute(self):
-        self.__x = float(self.node.get("x"))
-        self.__y = float(self.node.get("y"))
-        self.__center_x_to_item = float(self.node.get("centerXToItem"))
-        self.__center_y_to_item = float(self.node.get("centerYToItem"))
-        self.__center_x_to_global = float(self.node.get("centerXToGlobal"))
-        self.__center_y_to_global = float(self.node.get("centerYToGlobal"))
+        self.__x = int(round(float(self.node.get("x"))))
+        self.__y = int(round(float(self.node.get("y"))))
+        self.__center_x_to_item = int(round(float(self.node.get("centerXToItem"))))
+        self.__center_y_to_item = int(round(float(self.node.get("centerYToItem"))))
+        self.__center_x_to_global = int(round(float(self.node.get("centerXToGlobal"))))
+        self.__center_y_to_global = int(round(float(self.node.get("centerYToGlobal"))))
         self.__z = int(self.node.get("z"))
-        self.__height = float(self.node.get("height"))
-        self.__width = float(self.node.get("width"))
+        self.__height = int(round(float(self.node.get("height"))))
+        self.__width = int(round(float(self.node.get("width"))))
         self.__tempId = self.node.get("tempID")
         self.__text = self.node.get("text")
         self.__objectName = self.node.get("objectName")
@@ -102,129 +103,129 @@ class Item:
         self.__has_contents = bool(int(self.node.get("hasContents")))
 
     def __init_attribute(self):
-        self.__text = ""
-        self.__tempId = ""
-        self.__x = 0.0
-        self.__y = 0.0
-        self.__z = 0
-        self.__center_x_to_item = 0.0
-        self.__center_y_to_item = 0.0
-        self.__center_x_to_global = 0.0
-        self.__center_y_to_global = 0.0
-        self.__height = 0.0
-        self.__width = 0.0
-        self.__className = ""
-        self.__objectName = ""
-        self.__opacity = 0.0
-        self.__focus = None
-        self.__enabled = None
-        self.__visible = None
-        self.__scale = 0.0
-        self.__rotation = 0
-        self.__clip = None
-        self.__has_contents = None
+        self.__text = str()
+        self.__tempId = str()
+        self.__x = int()
+        self.__y = int()
+        self.__z = int()
+        self.__center_x_to_item = int()
+        self.__center_y_to_item = int()
+        self.__center_x_to_global = int()
+        self.__center_y_to_global = int()
+        self.__height = int()
+        self.__width = int()
+        self.__className = str()
+        self.__objectName = str()
+        self.__opacity = float()
+        self.__focus = bool()
+        self.__enabled = bool()
+        self.__visible = bool()
+        self.__scale = float()
+        self.__rotation = int()
+        self.__clip = bool()
+        self.__has_contents = bool()
 
-    def x(self, refresh=False):
+    def x(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__x
 
-    def y(self, refresh=False):
+    def y(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__y
 
-    def center_x_to_item(self, refresh=False):
+    def center_x_to_item(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__center_x_to_item
 
-    def center_y_to_item(self, refresh=False):
+    def center_y_to_item(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__center_y_to_item
 
-    def center_x_to_global(self, refresh=False):
+    def center_x_to_global(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__center_x_to_global
 
-    def center_y_to_global(self, refresh=False):
+    def center_y_to_global(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__center_y_to_global
 
-    def z(self, refresh=False):
+    def z(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__z
 
-    def scale(self, refresh=False):
+    def scale(self, refresh: bool = False) -> float:
         if refresh:
             self.__refresh_node()
         return self.__scale
 
-    def rotation(self, refresh=False):
+    def rotation(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__rotation
 
-    def clip(self, refresh=False):
+    def clip(self, refresh: bool = False) -> bool:
         if refresh:
             self.__refresh_node()
         return self.__clip
 
-    def height(self, refresh=False):
+    def height(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__height
 
-    def width(self, refresh=False):
+    def width(self, refresh: bool = False) -> int:
         if refresh:
             self.__refresh_node()
         return self.__width
 
-    def temp_id(self, refresh=False):
+    def temp_id(self, refresh: bool = False) -> str:
         if refresh:
             self.__refresh_node()
         return self.__tempId
 
-    def text(self, refresh=False):
+    def text(self, refresh: bool = False) -> str:
         if refresh:
             self.__refresh_node()
         return self.__text
 
-    def object_name(self, refresh=False):
+    def object_name(self, refresh: bool = False) -> str:
         if refresh:
             self.__refresh_node()
         return self.__objectName
 
-    def class_name(self, refresh=False):
+    def class_name(self, refresh: bool = False) -> str:
         if refresh:
             self.__refresh_node()
         return self.__className
 
-    def opacity(self, refresh=False):
+    def opacity(self, refresh: bool = False) -> float:
         if refresh:
             self.__refresh_node()
         return self.__opacity
 
-    def enabled(self, refresh=False):
+    def enabled(self, refresh: bool = False) -> bool:
         if refresh:
             self.__refresh_node()
         return self.__enabled
 
-    def visible(self, refresh=False):
+    def visible(self, refresh: bool = False) -> bool:
         if refresh:
             self.__refresh_node()
         return self.__visible
 
-    def focus(self, refresh=False):
+    def focus(self, refresh: bool = False) -> bool:
         if refresh:
             self.__refresh_node()
         return self.__focus
 
-    def exist(self, timeout=None):
+    def exist(self, timeout: int = None) -> bool:
         if timeout is None:
             timeout = self.device.default_timeout
         die_time = int(time.time()) + timeout
@@ -616,22 +617,23 @@ class Item:
         image = cv2.add(image, src2)
         return image
 
-    def click(self, delay=0):
+    def click(self, delay: int = 0) -> bool:
         if self.node is not None:
-            self.device.click(self.__center_x_to_global, self.__center_y_to_global, delay)
+            self.device.click(Point(self.__center_x_to_global, self.__center_y_to_global), delay)
+        return False
 
-    def click_exist(self, timeout=None):
+    def click_exist(self, timeout: int = None) -> bool:
         if self.exist(timeout):
             if (self.rect[len(self.rect) - 1][1] - self.rect[0][1] + 1) \
                     * (self.rect[len(self.rect) - 1][0] - self.rect[0][0] + 1) == len(self.rect):
                 x = self.rect[0][1] + int((self.rect[len(self.rect) - 1][1] - self.rect[0][1] + 1) / 2)
                 y = self.rect[0][0] + int((self.rect[len(self.rect) - 1][0] - self.rect[0][0] + 1) / 2)
-                self.device.click(x, y)
+                self.device.click(Point(x, y))
                 return True
-            self.device.click(self.rect[100][1], self.rect[100][0])
+            self.device.click(Point(self.rect[100][1], self.rect[100][0]))
             return True
         return False
 
-    def submit_string(self, text):
+    def submit_string(self, text: str) -> bool:
         self.click()
-        self.device.submit_string(text)
+        return self.device.submit_string(text)
