@@ -72,7 +72,7 @@ class Watcher:
         self.__watcher_data = data
         self.device = d
 
-    def when(self, sop_id, xpath_key):
+    def when(self, sop_id: str, xpath_key: str):
         if 'xpath_list' not in self.__watcher_data.keys():
             self.__watcher_data['xpath_list'] = []
         index = len(self.__watcher_data['xpath_list'])
@@ -91,7 +91,7 @@ class _WatchContext:
         self.__watcher_data = data
         self.device = d
 
-    def when(self, sop_id, xpath_key):
+    def when(self, sop_id: str, xpath_key: str):
         if 'xpath_list' not in self.__watcher_data.keys():
             self.__watcher_data['xpath_list'] = []
         index = len(self.__watcher_data['xpath_list'])
@@ -101,21 +101,21 @@ class _WatchContext:
         self.__watcher_data['index'] = index
         return self
 
-    def click(self):
+    def click(self) -> None:
         self.__watcher_data['active'] = 'click'
         self.__push()
 
-    def pause(self, key: str):
+    def pause(self, key: str) -> None:
         self.__watcher_data['active'] = key
         self.__push()
 
-    def launch(self, sop_id: str, ui_app_id: str):
+    def launch(self, sop_id: str, ui_app_id: str) -> None:
         self.__watcher_data['active'] = 'launch'
         self.__watcher_data['active_sop_id'] = sop_id
         self.__watcher_data['active_ui_app_id'] = ui_app_id
         self.__push()
 
-    def __push(self):
+    def __push(self) -> None:
         self.device.watcher_list.append(self.__watcher_data)
         self.device.push_watcher_data()
 
